
#include "nrf.h"
#include <arm_math.h> // CMSIS DSP
#include "nrf52_adc.h"

//#define DEUX_VOIES // décommenter pour tester 2 voies
//#define FFT // décommenter pour faire l'analyse spectrale

#define NSAMPLES 512 // nombre d'échantillons pour chaque voie
#ifdef DEUX_VOIES
#define NSAMPLES_16BITS 1024
#else
#define NSAMPLES_16BITS NSAMPLES
#endif

NRF52_ADC adc;
int16_t data[NSAMPLES_16BITS];
uint8_t nvoies;
float32_t data_f32[NSAMPLES];
arm_rfft_fast_instance_f32 rfft_instance;
float32_t fft[NSAMPLES*2];
float32_t fft_amp[NSAMPLES];

    	    
void setup() {
  Serial.begin(115200);
  for (int i=0; i<NSAMPLES; i++) data[i] = 0;
  
#ifdef DEUX_VOIES
  nvoies = 2;
  uint8_t voies[2] = {0,1};
  uint8_t gains[2] = {NRF52_ADC_GAIN_1_4,NRF52_ADC_GAIN_1_4};
  adc.config_multi_voies(nvoies,voies,gains,data,NSAMPLES,NRF52_ADC_RESOLUTION_12);
#else
  nvoies = 1;
  adc.config_une_voie(0,NRF52_ADC_GAIN_1_4,NRF52_ADC_OVERSAMPLE_NONE,data,NSAMPLES,NRF52_ADC_RESOLUTION_12);
#endif
  
  adc.config_echant(4,100); // échantillonnage à 10 kHz
}

    	     
void loop() {
  float f;
  float a;
  uint16_t indice=0;
  uint8_t voie;
  float conv_a = adc.get_ampl_conv(); // facteur de conversion nombre->volt
  float deltaf = adc.get_fechant()/NSAMPLES; // résolution en fréquence
  
  //adc.declencher();
  adc.declencher(7,NRF52_COMP_REF_2_4,32,32,1); // déclenchement par la borne A7
  // on peut faire autre chose ici, pendant que l'ADC travaille
  adc.attendre();
  for (indice=0; indice<NSAMPLES; indice++) data_f32[indice] = data[indice*nvoies+voie]*conv_a;
  delay(500);
  
#ifndef FFT
     for (indice=0; indice<NSAMPLES; indice++) Serial.println(data_f32[indice]);
#else
    arm_rfft_fast_init_f32(&rfft_instance,NSAMPLES);
    arm_rfft_fast_f32(&rfft_instance,data_f32,fft,0);
    arm_cmplx_mag_f32(fft,fft_amp,NSAMPLES);
    
    Serial.println("----------------");
    for (int i=0; i<NSAMPLES; i++) {
      a = fft_amp[i]*2/NSAMPLES;
      f = i*deltaf;
      if (a > 0.3) {
          Serial.print(f);
          Serial.print(" Hz, ");
          Serial.println(a);
      }
    }
#endif 
 
}   	     
    	     